# Copyright 1999 The Regents of the University of California
# All Rights Reserved
# 
# Permission to use, copy, modify and distribute any part of the
# GeoPlot tool for educational, research and non-profit
# purposes, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph
# and the following paragraphs appear in all copies.
# 
# Those desiring to incorporate this into commercial products or use
# for commercial purposes should contact the Technology Transfer
# Office, University of California, San Diego, 9500 Gilman Drive, La
# Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
# PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
# DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
# 
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
# UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
# OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
# OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
# PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
# ANY PATENT, TRADEMARK OR OTHER RIGHTS.
# 
# The GeoPlot tool is developed by Ram Periakaruppan at the University 
# of California, San Diego under the Cooperative Association for Internet 
# Data Analysis (CAIDA) Program. Support for this effort is provided by 
# the CAIDA grant NCR-9711092, and by CAIDA members.
# 
# Report bugs and suggestions to geoplot@caida.org.
package Key;

use Carp;

@ISA =  qw();
use strict;

use vars qw($AUTOLOAD);

sub new 
{
     my $this = shift;
     my ($x, $y, $width, $height, $title, $tfsize, @data_sets) = @_;
     my $data_ref;
     my ($range, $value, $desc, $divisions, $keyline) = ("","","","","");

     my $class = ref($this) || $this;
     my $self = {
		  'x'           => undef,
		  'y'           => undef,
		  'width'       => undef,
		  'height'      => undef,
		  'title'       => undef,
		  'tfsize'      => undef,
		  'data_ranges' => {},
		};

     bless $self, $class;
     
     $self->{'x'} = $x;
     $self->{'y'} = $y;
     $self->{'width'} = $width;
     $self->{'height'} = $height;
     $self->{'title'} = $title;
     $self->{'tfsize'} = $tfsize;
     
     $data_ref = shift @data_sets;

     if (ref($data_ref) ne "ARRAY") 
     {
	 carp "Data set must be an array";
	 return 0;
     }
     $self->{'data_ranges'} = $data_ref;
     
     return $self;
}

sub AUTOLOAD
{
     my $self = shift;
     my $type = ref($self) || croak "$self is not an object";
     my $name = $AUTOLOAD;
     $name =~ s/.*://;
     unless (exists $self->{$name} )
     {
	 croak "Can't access $name field in object of class $type";
     }
     
     if (@_)
     {
	 return $self->{$name} = shift;
     }
     else
     {
	 return $self->{$name};
     }
}

sub print
{
     my $self = shift;
     my ($range, $value, $desc, $divisions, $keyline) = ("","","","","");
     my $keytoString = "\t\tkey,x:" . $self->{'x'} . ",y:" . $self->{'y'} 
                       . ",w:" . $self->{'width'} . ",h:" . $self->{'height'}
                       . ",t:" . $self->{'title'} . ",s:" . $self->{'tfsize'}
                       . ";\n";
    
     my $array_len = @{$self->{'data_ranges'}};
     for (my $i=0; $i < $array_len; $i++)
     {
	 $range = ${@{$self->{'data_ranges'}}[$i]}{"range"};
         $value = ${@{$self->{'data_ranges'}}[$i]}{"value"};
         $desc = ${@{$self->{'data_ranges'}}[$i]}{"desc"};
 
         if ($range && $value)
         {
	     if ($desc)
	     {
		 $keyline = $value." : ".$desc;
	     }
	     else
	     {
		 $keyline = $value;
	     }
             $keytoString .= "\t\t" . $range . " : " . $keyline .";\n";
         }
     }
   
     $keytoString .= "\t   \">\n\n";
     return $keytoString;
}

1;
