# Copyright 1999 The Regents of the University of California
# All Rights Reserved
# 
# Permission to use, copy, modify and distribute any part of the
# GeoPlot tool for educational, research and non-profit
# purposes, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph
# and the following paragraphs appear in all copies.
# 
# Those desiring to incorporate this into commercial products or use
# for commercial purposes should contact the Technology Transfer
# Office, University of California, San Diego, 9500 Gilman Drive, La
# Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
# PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
# DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
# 
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
# UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
# OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
# OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
# PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
# ANY PATENT, TRADEMARK OR OTHER RIGHTS.
# 
# The GeoPlot tool is developed by Ram Periakaruppan at the University 
# of California, San Diego under the Cooperative Association for Internet 
# Data Analysis (CAIDA) Program. Support for this effort is provided by 
# the CAIDA grant NCR-9711092, and by CAIDA members.
# 
# Report bugs and suggestions to geoplot@caida.org.

package Node;

use Carp;

@ISA =  qw();
use strict;


use vars qw($AUTOLOAD);

sub new 
{
     my $this = shift;
     my ($name,$lat,$lon,%node_attributes) = @_;
     my $class = ref($this) || $this;

     my $self = {
		 'name' => undef,
		 'lat' => undef,       
		 'lon' => undef,       
		 'title' => "",         
		 'size' => "",          
		 'value_size' => "",     
		 'value_color' => "",
		 'color' => "",
		 'url' => "",
		 'desc' => "",               
     };

     bless $self, $class;

     $self->{'name'} = $name;
     $self->{'lat'} = $lat;
     $self->{'lon'} = $lon;

     if ($node_attributes{"title"})
     {
	 $self->{'title'} = $node_attributes{"title"};    
     }
     
     if ($node_attributes{"size"})
     {
	 $self->{'size'} = $node_attributes{"size"};
     }
     
     if ($node_attributes{"vsize"})
     {
	 $self->{'value_size'} = $node_attributes{"vsize"};
     }
     
     if ($node_attributes{"vcolor"})
     {
	 $self->{'value_color'} = $node_attributes{"vcolor"};
     }

     if ($node_attributes{"color"})
     {
	 $self->{'color'} = $node_attributes{"color"};
     }
     
     if ($node_attributes{"url"})
     {
	 $self->{'url'} = $node_attributes{"url"};
     }
	
     if ($node_attributes{"desc"})
     {
	 $self->{'desc'} = $node_attributes{"desc"};
     }

     return $self;
}

sub AUTOLOAD
{
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{$name} )
    {
	croak "Can't access $name field in object of class $type";
    }
	
    if (@_)
    {
	return $self->{$name} = shift;
    }
    else
    {
	return $self->{$name};
    }
}

sub print
{
    my $self = shift;
    my $nodetoString = "n " . $self->{'name'} . " " . $self->{'lat'} 
                       . " " . $self->{'lon'};

    if ($self->{'title'} ne "")
    {
	$nodetoString .= " t:" . $self->{'title'};
    }

    if ($self->{'size'} ne "")
    {
	$nodetoString .= " sz:" . $self->{'size'};
    }

    if ($self->{'value_size'} ne "")
    {
	$nodetoString .= " vs:" . $self->{'value_size'};
    }

    if ($self->{'value_color'} ne "")
    {
	$nodetoString .= " vc:" . $self->{'value_color'};
    }

    if ($self->{'color'} ne "")
    {
	$nodetoString .= " cl:" . $self->{'color'};
    }

    if ($self->{'url'} ne "")
    {
	$nodetoString .= " u:" . $self->{'url'};
    }

    if ($self->{'desc'} ne "")
    {
	$nodetoString .= " d:" . $self->{'desc'};
    }

    $nodetoString .= ";\n";
    return $nodetoString;
}

1;
