# Copyright 1999 The Regents of the University of California
# All Rights Reserved
# 
# Permission to use, copy, modify and distribute any part of the
# GeoPlot tool for educational, research and non-profit
# purposes, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph
# and the following paragraphs appear in all copies.
# 
# Those desiring to incorporate this into commercial products or use
# for commercial purposes should contact the Technology Transfer
# Office, University of California, San Diego, 9500 Gilman Drive, La
# Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
# PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
# DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
# 
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
# UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
# OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
# OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
# PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
# ANY PATENT, TRADEMARK OR OTHER RIGHTS.
# 
# The GeoPlot tool is developed by Ram Periakaruppan at the University 
# of California, San Diego under the Cooperative Association for Internet 
# Data Analysis (CAIDA) Program. Support for this effort is provided by 
# the CAIDA grant NCR-9711092, and by CAIDA members.
# 
# Report bugs and suggestions to geoplot@caida.org.

package Path;

use Carp;

@ISA =  qw();
use strict;

use vars qw($AUTOLOAD);

sub new 
{
     my $this = shift;
     my %path_attributes = ();
     my ($path_attributes_ref, $nodes_ref) = @_;
     my $class = ref($this) || $this;
     
     my $self = { 
		  'p_nodes' => undef,
		  'title' => "",         
		  'size' => "",          
		  'value_size' => "",     
		  'value_color' => "",
		  'color' => "",
		  'url' => "",
		  'desc' => "",      
		  'direc' => "",              
		};

     bless $self, $class;

     $self->{'p_nodes'} = $nodes_ref;
     %path_attributes = %{$path_attributes_ref};

     if ($path_attributes{"title"})
     {
	 $self->{'title'} = $path_attributes{"title"};    
     }
     
     if ($path_attributes{"size"})
     {
	 $self->{'size'} = $path_attributes{"size"};
     }
     
     if ($path_attributes{"vsize"})
     {
	 $self->{'value_size'} = $path_attributes{"vsize"};
     }
     
     if ($path_attributes{"vcolor"})
     {
	 $self->{'value_color'} = $path_attributes{"vcolor"};
     }

     if ($path_attributes{"color"})
     {
	 $self->{'color'} = $path_attributes{"color"};
     }
     
     if ($path_attributes{"url"})
     {
	 $self->{'url'} = $path_attributes{"url"};
     }
	
     if ($path_attributes{"desc"})
     {
	 $self->{'desc'} = $path_attributes{"desc"};
     }

     if ($path_attributes{"direc"})
     {
	 if ($path_attributes{"direc"} eq "dest")
	 {
	     $self->{'direc'} = 1;
	 }
	 elsif ($path_attributes{"direc"} eq "src")
	 {
	     $self->{'direc'} = 2;
	 }
	 if ($path_attributes{"direc"} eq "srcdest")
	 {
	     $self->{'direc'} = 3;
	 }
     }

     return $self;
}

sub AUTOLOAD
{
	my $self = shift;
	my $type = ref($self) || croak "$self is not an object";
	my $name = $AUTOLOAD;
	$name =~ s/.*://;
	unless (exists $self->{$name} )
	{
		croak "Can't access $name field in object of class $type";
	}
	
	if (@_)
	{
		return $self->{$name} = shift;
	}
	else
	{
		return $self->{$name};
	}
}

sub get_attributes
{
    my $self = shift;
    my %link_attributes =
	(
	    "title",$self->{'title'},         
	    "size",$self->{'size'},          
	    "vsize",$self->{'value_size'},     
	    "vcolor", $self->{'value_color'},
	    "color",$self->{'color'},
	    "url",$self->{'url'},
	    "desc",$self->{'desc'},      
	    "direc",$self->{'direc'},
        );             

    #print "link_attributes ", $link_attributes{"direc"}, "\n";
    return %link_attributes;
}


sub print
{
     my $self = shift;
     my $array_len = $#{$self->{'p_nodes'}};
     my ($src, $dest, $pathtoString) = ("","","");

     for (my $i=0; $i < $array_len; $i++)
     {
	 $src = $self->{'p_nodes'}[$i];
	 $dest = $self->{'p_nodes'}[$i+1];

	 if (ref($src) eq "Node")
	 {
	    $src = $src->name;
	 }
	 if (ref($dest) eq "Node")
	 {
	    $dest = $dest->name;
	 }

	 $pathtoString .= "\t\tl " . $src . " " . $dest;

	 if ($self->{'title'} ne "")
	 {
	     $pathtoString .= " t:" . $self->{'title'};
	 }

	 if ($self->{'size'} ne "")
	 {
	     $pathtoString .= " sz:" . $self->{'size'};
	 }

	 if ($self->{'value_size'} ne "")
	 {
	     $pathtoString .= " vs:" . $self->{'value_size'};
	 }
	 
	 if ($self->{'value_color'} ne "")
	 {
	     $pathtoString .= " vc:" . $self->{'value_color'};
	 }
 
	 if ($self->{'color'} ne "")
	 {
	     $pathtoString .= " cl:" . $self->{'color'};
	 }

	 if ($self->{'url'} ne "")
	 {
	     $pathtoString .= " u:" . $self->{'url'};
	 }

	 if ($self->{'desc'} ne "")
	 {
	     $pathtoString .= " d:" . $self->{'desc'};
	 }
	 
	 if ($self->{'direc'} ne "")
	 {
	     $pathtoString .= " r:" . $self->{'direc'};
	 }

	 $pathtoString .= ";\n";
     }
     
     return $pathtoString;
}

1;
